#include "stdafx.h"

#include "office/ExcelWrapper.h"

namespace MSO::Excel {

ExcelWrapper::~ExcelWrapper() {
  if (m_NeedClose && IsActive()) {
    try {
      xlApplication.Quit();
    } catch (...) {
      return;
    }
  }
}

void ExcelWrapper::Clear() {
  linkedWb.ReleaseDispatch();
  xlApplication.ReleaseDispatch();
  m_NeedClose = TRUE;
}

BOOL ExcelWrapper::InitApplication(BOOL isVisible) {
  if (!IsActive() && !xlApplication.CreateDispatch(L"Excel.Application")) {
    return FALSE;
  } else {
    xlApplication.put_Visible(isVisible);
    return TRUE;
  }
}

EXLWorkbook ExcelWrapper::OpenDocument(const CString docPath, BOOL docVisible,
                                                BOOL docReadOnly, BOOL closeAfter, BOOL add2Recent) {
  if (!::PathFileExistsW(docPath)) {
    return nullptr;
  } else if (!InitApplication(docVisible)) {
    return nullptr;
  } else {
    m_NeedClose = closeAfter;
    COleVariant readOnly{ static_cast<long>(docReadOnly) };
    COleVariant addMRU{ static_cast<long>(add2Recent) };
    linkedWb = xlApplication.get_Workbooks().Open(_bstr_t{ docPath }, vtMissing, readOnly, vtMissing, vtMissing,
                                                  vtMissing, vtMissing, vtMissing, vtMissing, vtMissing,
                                                  vtMissing, vtMissing, addMRU, vtMissing, vtMissing);
    return linkedWb;
  }
}

EXLWorkbook ExcelWrapper::NewDocument(const CString docTemplate, BOOL closeAfter) {
  if (!InitApplication(TRUE)) {
    return nullptr;
  } else {
    auto templatePath = docTemplate;
    if (docTemplate.IsEmpty() || !::PathFileExistsW(docTemplate))
      templatePath.Empty();

    m_NeedClose = closeAfter;
    COleVariant vPath{ templatePath };
    linkedWb = xlApplication.get_Workbooks().Add(vPath);
    return linkedWb;
  }
}

void ExcelWrapper::ReleaseDocumentAndQuit() {
  m_NeedClose = TRUE;
  ReleaseDocument();
}

void ExcelWrapper::ReleaseDocument() {
  if (m_NeedClose) {
    CloseApplication();
  }
  Clear();
}

void ExcelWrapper::CloseApplication() {
  if (IsActive()) {
    xlApplication.Quit();
  }
}

EXLWorkbook ExcelWrapper::GetDocument() {
  return linkedWb;
}

CString ExcelWrapper::PromptUserForPath(const CString text) {
  CFileDialog dialog(TRUE, 0, LR"(*.xlsx)", 0, text);
  if (dialog.DoModal() != IDOK) {
    return{};
  } else {
    return dialog.GetPathName();
  }
}

} // namespace MSO::Excel